% 2007 nov 9, demo to match moments with gaussian

% Matching KL(p||q) with q(x) = exp(a*x^2 + b*x + c).

pfunc = @(x) abs(cos(x) + 3*sin(7*x)).*exp(-(x-1).^4);
logpfunc = @(x) log( abs(cos(x) + 3*sin(7*x)) + eps ) - (x-1).^4;
domain = -10:0.05:10;

pval = pfunc(domain);
logpval = logpfunc(domain);

%normalize ? a degenerate case
%pval = pval/numericalMoments(pval,domain,0);

nu = numericalFitGaussianMoments012(pval, domain)

qval = exp( nu(1) + (nu(2) * domain) + (nu(3) * domain.^2));
numericalMoments(pval, domain, [0,1,2])
numericalMoments(qval, domain, [0,1,2])

figure, hold on, plot(domain, pval, 'blue'), plot(domain, qval, 'red')

% Let's have fun with the alpha divergence, with iterations.
% Assumes that the above code has been run.

alpha = 0.3;

if(alpha == 0)
    error('This iterative scheme does not work for alpha = 0\n');
end

for s=1:10
    paqma = (pval.^alpha) .* (qval.^(1-alpha));

    nu = numericalFitGaussianMoments012(paqma, domain)
    qval = exp( nu(1) + (nu(2) * domain) + (nu(3) * domain.^2));
end

figure, hold on, plot(domain, pval, 'blue'), plot(domain, qval, 'red')


% Let's allow alpha outside of [0,1] by using the values of nu instead of qval.

epsilon = 0.5;
alpha = 3;

for s=1:10
    paqma = (pval.^alpha) .* exp((1-alpha)*(nu(1) + (nu(2) * domain) + (nu(3) * domain.^2)));

    newnu = numericalFitGaussianMoments012(paqma, domain);
    nu = nu * epsilon + newnu * (1-epsilon)
    qval = exp( nu(1) + (nu(2) * domain) + (nu(3) * domain.^2));
end

figure, hold on, plot(domain, pval, 'blue'), plot(domain, qval, 'red')

% Now let's use the log of p to try to have negative values of alpha.
% This is busting out of proportions every time ...

epsilon = 0.5;
alpha = -0.2;

for s=1:10
    paqma = exp( alpha*logpval + (1-alpha)*(nu(1) + (nu(2) * domain) + (nu(3) * domain.^2)));
    
    newnu = numericalFitGaussianMoments012(paqma, domain);
    nu = nu * epsilon + newnu * (1-epsilon)
    qval = exp( nu(1) + (nu(2) * domain) + (nu(3) * domain.^2));
end

figure, hold on, plot(domain, pval, 'blue'), plot(domain, qval, 'red')
